// Program Is PropLoad.c
// Last Revision On 25Feb20

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <propeller.h>
#include <catalina_rtc.h>


#define     SDA_Pin                     29
#define     SCL_Pin                     28

#define     SDA_Hi                      1 << SDA_Pin
#define     SCL_Hi                      1 << SCL_Pin

#define     SDA_Lo                      SDA_Hi ^ 0xffffffff
#define     SCL_Lo                      SCL_Hi ^ 0xffffffff

struct Memee
{
 unsigned int A00     :1;
 unsigned int A01     :1;
 unsigned int A02     :1;
 unsigned int A03     :1;
 unsigned int A04     :1;
 unsigned int A05     :1;
 unsigned int A06     :1;
 unsigned int A07     :1;

 unsigned int A08     :1;
 unsigned int A09     :1;
 unsigned int A10     :1;
 unsigned int A11     :1;
 unsigned int A12     :1;
 unsigned int A13     :1;
 unsigned int A14     :1;
 unsigned int A15     :1;

 unsigned int A16     :1;
 unsigned int A17     :1;
 unsigned int A18     :1;
 unsigned int A19     :1;
 unsigned int A20     :1;
 unsigned int A21     :1;
 unsigned int A22     :1;
 unsigned int A23     :1;

 unsigned int RW      :1;
 unsigned int E16     :1;
 unsigned int E17     :1;
 unsigned int E18     :1;
 unsigned int Device  :4;
};

union Morpee
{
 struct   Memee     DevAdd;
          int       Address;
          char      Byte[4];
};

union Morpee EEPROM;
int Address=0;
int Value;
char InStr[512];
char OutStr[512];


void i2c_time_delay(int ticks)
{
 WAIT(ticks);
}

void i2c_sda_input(void)
{
 DIRA &= SDA_Lo;
}

void i2c_sda_output(void)
{
 DIRA |= SDA_Hi;
}

void i2c_init(void)
{
 OUTA |= SCL_Hi;                   // SCL=Hi
 OUTA |= SDA_Hi;                   // SDA=Hi
 DIRA |= SCL_Hi;                   // Set SCL As Output
 DIRA |= SDA_Hi;                   // Set SDA As Output
}

void i2c_scl_hi(void)
{
 DIRA |= SCL_Hi;
 OUTA |= SCL_Hi;
}

void i2c_scl_lo(void)
{
 DIRA |= SCL_Hi;
 OUTA &= SCL_Lo;
}

void i2c_sda_hi(void)
{
 DIRA |= SDA_Hi;
 OUTA |= SDA_Hi;
}

void i2c_sda_lo(void)
{
 DIRA |= SDA_Hi;
 OUTA &= SDA_Lo;
}

void i2c_start(void)
{
 i2c_sda_lo();                     // SDA=Lo
 i2c_time_delay(528);
 i2c_scl_lo();                     // SCL=Lo
}

void i2c_stop(void)
{
 i2c_scl_hi();                     // SCL=Hi
 i2c_time_delay(528);
 i2c_sda_hi();                     // SDA=Hi
}

void i2c_reset(void)
{
 char index;
 i2c_start();
 for(index=0; index<9; index++)
  {
   i2c_scl_hi();
   i2c_time_delay(528);
   i2c_scl_lo();
   i2c_time_delay(528);
  }
 i2c_start();
 i2c_stop();
}

char i2c_byte_write(char Byte)
{
 char result;
 char index;
 char mask=0x80;
 for(index=0; index<8; index++)
  {
   result=mask & Byte;
   if(result > 0) i2c_sda_hi();
   else i2c_sda_lo();
   mask=mask >> 1;
   i2c_time_delay(264);
   i2c_scl_hi();
   i2c_time_delay(264);
   i2c_scl_lo();
   if(index == 7) i2c_sda_input();
   i2c_time_delay(264);
  }
 i2c_time_delay(132);
 i2c_scl_hi();
 i2c_time_delay(396);
 result=(char) ((INA & SDA_Hi) >> SDA_Pin); 
 i2c_scl_lo();
 if(result == 0) i2c_sda_lo();
 i2c_time_delay(264);
 return(result);
}

char EEWrite(void)
{
 int Index;
 char HiMem;
 EEPROM.Address=Address;
 EEPROM.DevAdd.Device=0x0a;
 EEPROM.DevAdd.RW=0;
 EEPROM.DevAdd.E16=EEPROM.DevAdd.A16;
 EEPROM.DevAdd.E17=EEPROM.DevAdd.A17;
 EEPROM.DevAdd.E18=EEPROM.DevAdd.A18;
 HiMem=(EEPROM.DevAdd.E18 << 2) | (EEPROM.DevAdd.E17 << 1) | EEPROM.DevAdd.E16;
 i2c_start();
 if(i2c_byte_write(EEPROM.Byte[3]) != 0x00) return('N');
 if(i2c_byte_write(EEPROM.Byte[1]) != 0x00) return('N');
 if(i2c_byte_write(EEPROM.Byte[0]) != 0x00) return('N');
 for(Index=0; Index<Value; Index++) if(i2c_byte_write(InStr[Index]) != 0x00) return('N');
 i2c_stop();
 return('Y');
}

void main(void)
{
 int InPut;
 char AorD;
 char RxD;
 char TxD;
 char Left;
 char Right;
 char Data;
 char Flip=0;
 i2c_init();
 for(;;)
  {
   InPut=tty_rxcheck();
   if(InPut == -1) continue;
   RxD=(char) InPut;
   switch(RxD)
    {
     case '0':
     case '1':
     case '2':
     case '3':
     case '4':
     case '5':
     case '6':
     case '7':
     case '8':
     case '9':
     case 'A':
     case 'B':
     case 'C':
     case 'D':
     case 'E':
     case 'F': TxD=RxD;
               if(RxD >= 'A') RxD=RxD - 'A' + 0x0a;
               else RxD=RxD - '0';
               switch(AorD)
                {
                 case 1: Address=(Address << 4) | RxD;
                         break;
                 case 0: switch(Flip)
                          {
                           case 0: Data=RxD << 4;
                                   Flip=1;
                                   break;
                           case 1: Data=Data | RxD;
                                   InStr[Value++]=Data;
                                   Flip=0;
                                   break;
                          }
                         break;
                 }
               tty_tx(TxD);
               break;
     case 'M': AorD=1;
               Address=0;
               tty_tx('M');
               break;
     case 'L': AorD=0;
               Flip=0;
               Value=0;
               tty_tx('L');
               break;
     case 'W': if(EEWrite() == 'Y') tty_tx('W');
               else tty_tx('T');
               break;
     case '?': tty_tx('P');
     default:  break;
    }
  }
}

